#include "mex_interface.h"
#include "mex_convert.h"

#include "greedy_clustering.h"



#ifndef INFINITY
#define INFINITY (1.0/0.0)
#endif

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  size_t n, ndims, k, dims[2];
  const double *p;
  double *C, mindist;
  int ki, *part;

  mexPrintf("Importing arguments...\n");

  /* Import arguments */
  mex_assert(nrhs >= 2 && nlhs <= 2);

  ndims = (size_t)mxGetM(prhs[0]);
  n = (size_t)mxGetN(prhs[0]);

  mex_assert(mxGetClassID(prhs[0]) == mxDOUBLE_CLASS);
  p = mxGetPr(prhs[0]);

  k = (size_t)mxArrayToIntScalar(prhs[1]);

  if(nrhs >= 3) {
    mindist = mxArrayToDoubleScalar(prhs[2]);
  }
  else {
    mindist = -INFINITY;
  }

  /* Run gonzalez */
  C = (double *)malloc(ndims * k * sizeof(double));
  part = (int *)malloc(n * sizeof(int));

  k = greedy_cluster(n, ndims, p, k, mindist, C, part);

  /* Output */
  dims[0] = 1;
  dims[1] = n;
  plhs[0] = mxArrayFromIntArray(2, dims, part);

  if(nlhs >= 2) {
    dims[0] = ndims;
    dims[1] = k;
    plhs[1] = mxArrayFromDoubleArray(2, dims, C);
  }
}
