#include <assert.h>

#include "kmeans_util.h"

#include "geom.h"
#include "sort.h"

void cluster_centers(
  size_t n, size_t ndims, const double *p, const double *w,
  size_t k, const int *part, double *C)
{
  size_t i, j, d;
  double obj, dist, *S;

  double *ones;
  if(w == NULL) {
    ones = ones_vector(n);
    cluster_centers(n, ndims, p, ones, k, part, C);
    free(ones);
    return;
  }

  S = (double *)malloc(sizeof(double) * k);

  // Find centers
  for(j = 0; j < k * ndims; j++) {
    C[j] = 0;
  }
  for(j = 0; j < k; j++) {
    S[j] = 0;
  }

  for(i = 0; i < n; i++) {
    j = (size_t)part[i];
    for(d = 0; d < ndims; d++) {
      C[ndims*j + d] += w[i] * p[ndims*i + d];
    }
    S[j] += w[i];
  }

  for(j = 0; j < k; j++) {
    geom_vector_scale(ndims, C + ndims*j, 1/S[j]);
  }

  free(S);
}

double *ones_vector(size_t n)
{
  double *v;
  size_t i;

  v = malloc(sizeof(double) * n);
  for(i = 0; i < n; i++) {
    v[i] = 1;
  }

  return v;
}

size_t normalize_part(size_t n, int *part)
{
  int *unique;
  size_t i, k;

  // Find unique labels
  unique = (int *)malloc(n * sizeof(int));
  memcpy(unique, part, n * sizeof(int));
  sort_ints(n, unique);

  k = 1;
  for(i = 1; i < n; i++) {
    if(unique[i] != unique[i-1]) {
      unique[k++] = unique[i];
    }
  }

  // Map to index into unique
  for(i = 0; i < n; i++) {
    part[i] = bsearch_ints(k, part[i], unique) - unique;
  }

  free(unique);

  return k;
}
