#include "sort.h"

#define SORT_TYPE(type)                                                 \
                                                                        \
  static int compare_ ## type(const void *p1, const void *p2)           \
  {                                                                     \
    type i1, i2;                                                        \
                                                                        \
    i1 = *(const type *)p1;                                             \
    i2 = *(const type *)p2;                                             \
                                                                        \
    if(i1 < i2) {                                                       \
      return -1;                                                        \
    }                                                                   \
    else if(i1 == i2) {                                                 \
      return 0;                                                         \
    }                                                                   \
    else {                                                              \
      return 1;                                                         \
    }                                                                   \
  }                                                                     \
                                                                        \
  void sort_ ## type ## s(size_t n, type *v)                            \
  {                                                                     \
    qsort(v, n, sizeof(type), &compare_ ## type);                       \
  }                                                                     \
                                                                        \
  typedef struct type ## pair {                                         \
    type key;                                                           \
    int value;                                                          \
  } type ## pair_t;                                                     \
                                                                        \
  static int compare_ ## type ## _pair(const void *p1, const void *p2)  \
  {                                                                     \
    const type *i1, *i2;                                                \
                                                                        \
    i1 = &((const type ## pair_t *)p1)->key;                            \
    i2 = &((const type ## pair_t *)p2)->key;                            \
                                                                        \
    return compare_ ## type(i1, i2);                                    \
  }                                                                     \
                                                                        \
                                                                        \
  void sort_ ## type ## s_permutation(size_t n, type *list, int *perm)  \
  {                                                                     \
    type ## pair_t *pairs;                                              \
    size_t i;                                                           \
                                                                        \
    /* Create pairing of keys and their original position */            \
    pairs = (type ## pair_t *)malloc(sizeof(type ## pair_t) * n);       \
    for(i = 0; i < n; i++) {                                            \
      pairs[i].key = list[i];                                           \
      pairs[i].value = (int)i;                                          \
    }                                                                   \
                                                                        \
    /* Sort */                                                          \
    qsort(pairs, n, sizeof(type ## pair_t), &compare_ ## type ## _pair); \
                                                                        \
    /* Save permutation */                                              \
    for(i = 0; i < n; i++) {                                            \
      list[i] = pairs[i].key;                                           \
      perm[i] = pairs[i].value;                                         \
    }                                                                   \
                                                                        \
    free(pairs);                                                        \
  }                                                                     \
                                                                        \
  type *bsearch_ ## type ## s(size_t n, type key, type *values)         \
  {                                                                     \
    void *p;                                                            \
                                                                        \
    p = bsearch(&key, values, n, sizeof(type), &compare_ ## type);      \
                                                                        \
    return (type *)p;                                                   \
  }


SORT_TYPE(int)
SORT_TYPE(double)

void unique_ints(size_t *out_n, int *list)
{
  size_t i, j, n;

  n = *out_n;
  if(n == 0) {
    return;
  }

  sort_ints(n, list);

  j = 1;
  for(i = 1; i < n; i++) {
    if(list[i] != list[i-1]) {
      list[j++] = list[i];
    }
  }

  *out_n = j;
}
